import type { IMenuItem } from "@interfaces/IMenu";
import type { MDXInstance } from "astro";
import type { IPage } from "@interfaces/IPage";
import { Home } from "@carbon/icons-react";
import React from "react";

interface Breadcrumb {
  title?: string;
  target?: string;
  icon?: React.ReactElement;
}

/**
 * Builds a menu structure from the Filesystem structure given by Astro.glob()
 */
export const buildPageTree = (
  rawContent: MDXInstance<IPage>[],
  basePath: string,
  locale?: string,
  ignoreHidden?: boolean
): IMenuItem[] => {
  const flatMenu: IMenuItem[] = [];

  // Build a sorted flat array first...
  rawContent
    .filter((item) => item.url)
    .sort((a, b) => (a?.url?.length || 0) - (b?.url?.length || 0))
    .forEach((item) => {
      if (item.url === basePath) return;
      const pathSegments =
        basePath === "/"
          ? item.url?.split("/").filter(Boolean)
          : item.url?.replace(basePath, "").split("/").filter(Boolean);

      // Now we're creating objects for every path segment of the pages in the flat array.
      // E.g. for the page /en/blog/my-blog-post, we create 3 entries `en`, `blog` and `my-blog-post`
      pathSegments?.forEach((_, index: number) => {
        const localizedKey = pathSegments.slice(0, index + 1).join("/") || "";
        const localizedParentKey =
          index > 0 ? pathSegments.slice(0, index).join("/") : null;

        if (
          index === pathSegments.length - 1 &&
          (!item.frontmatter.hidden || ignoreHidden)
        ) {
          flatMenu.push({
            key: localizedKey,
            parentKey: localizedParentKey,
            title: item.frontmatter.title,
            ...(index > 1
              ? item.frontmatter.category
                ? {}
                : { target: item.url }
              : item.frontmatter.hideChildren
              ? { target: item.url }
              : {}),
            order: item.frontmatter.order || 0,
            hideChildren:
              !ignoreHidden && (item.frontmatter.hideChildren ?? false),
          });
        }
      });
    });

  const filteredTopLevel = flatMenu.filter((ele) => !ele.parentKey);

  // Build tree root with top level (languages) as children
  const rootNode = {
    key: "root",
    children: filteredTopLevel,
  };

  const addChildren = (currentNode: IMenuItem) => {
    currentNode.children = flatMenu
      .filter((e) => e.parentKey === currentNode.key)
      .sort((a, b) => (a?.order || 0) - (b?.order || 0));

    for (const child of currentNode.children) {
      if (!child.hideChildren) {
        addChildren(child);
      }
    }
    currentNode.children = currentNode.children.map((e) => {
      // Remove unnecessary props that were only used for creating the nested structure
      const { parentKey, order, hideChildren, ...strippedProps } = e;
      return { ...strippedProps };
    });
  };

  for (const node of rootNode.children) {
    addChildren(node);
  }

  if (locale) {
    const languageSubtree = rootNode.children.find(
      (item) => item.key.toLowerCase() === locale?.toLowerCase()
    )?.children;

    return languageSubtree || [];
  }

  return rootNode.children;
};

const buildBreadcrumbsRaw = (
  itemTree: IMenuItem[],
  currentPath: string
): Array<Breadcrumb> | null => {
  for (const item of itemTree) {
    if (item.target === currentPath) {
      return [{ target: item.target, title: item.title }];
    } else {
      if (item.children) {
        const subItems = buildBreadcrumbsRaw(item.children, currentPath);
        if (subItems) {
          return [{ target: item.target, title: item.title }, ...subItems];
        }
      }
    }
  }
  return null;
};

const addBreadcrumbsHome = (
  breadcrumbs: Array<Breadcrumb> | null,
  baseUrl: string
) => {
  const homeLevel = {
    target: baseUrl,
    title: "Home",
    icon: <Home />,
  };
  if (breadcrumbs?.length) {
    const { target, ...lastLevelNoTarget } =
      breadcrumbs?.[breadcrumbs?.length - 1];
    return [homeLevel, ...(breadcrumbs?.slice(0, -1) || []), lastLevelNoTarget];
  }
  return [homeLevel, ...(breadcrumbs || [])];
};

export const buildBreadcrumbs = (
  itemTree: IMenuItem[],
  currentPath: string,
  baseUrl: string
): Array<Breadcrumb> | null => {
  return addBreadcrumbsHome(
    buildBreadcrumbsRaw(itemTree, currentPath),
    baseUrl
  );
};
